@@ -49,6 +49,10 @@ class TestHarness(object):
         print "PID:", os.getpid()
         print
         
+        # Override these in _run as needed:
+        cherrypy.server.using_wsgi = True
+        cherrypy.server.using_apache = False
+        
         if isinstance(conf, basestring):
             parser = cherrypy.config._Parser()
             conf = parser.dict_from_file(conf).get('global', {})
@@ -90,6 +94,7 @@ class CommandLineParser(object):
                          'cpmodpy': "cpmodpy",
                          'modpygw': "modpygw",
                          'modwsgi': "modwsgi",
+                         'modfcgid': "modfcgid",
                          }
     default_server = "wsgi"
     scheme = "http"
@@ -332,6 +337,11 @@ class CommandLineParser(object):
                                            self.protocol, self.port,
                                            "http", self.interactive)
             h.use_wsgi = True
+        elif self.server == 'modfcgid':
+            from cherrypy.test import modfcgid
+            h = modfcgid.FCGITestHarness(self.tests, self.server,
+                                         self.protocol, self.port,
+                                         "http", self.interactive)
         else:
             h = TestHarness(self.tests, self.server, self.protocol,
                             self.port, self.scheme, self.interactive,
