@@ -7,6 +7,7 @@ import os
 localDir = os.path.dirname(__file__)
 import sys
 import types
+from httplib import IncompleteRead
 
 import cherrypy
 from cherrypy import _cptools, tools
@@ -16,7 +17,7 @@ from cherrypy.lib import http, static
 favicon_path = os.path.join(os.getcwd(), localDir, "../favicon.ico")
 
 defined_http_methods = ("OPTIONS", "GET", "HEAD", "POST", "PUT", "DELETE",
-                        "TRACE", "CONNECT", "PROPFIND")
+                        "TRACE", "PROPFIND")
 
 
 def setup_server():
@@ -98,6 +99,40 @@ def setup_server():
         def default(self, *args, **kwargs):
             return "args: %s kwargs: %s" % (args, kwargs)
 
+    class ParamErrors(Test):
+
+        def one_positional(self, param1):
+            return "data"
+        one_positional.exposed = True
+
+        def one_positional_args(self, param1, *args):
+            return "data"
+        one_positional_args.exposed = True
+
+        def one_positional_args_kwargs(self, param1, *args, **kwargs):
+            return "data"
+        one_positional_args_kwargs.exposed = True
+
+        def one_positional_kwargs(self, param1, **kwargs):
+            return "data"
+        one_positional_kwargs.exposed = True
+
+        def no_positional(self):
+            return "data"
+        no_positional.exposed = True
+
+        def no_positional_args(self, *args):
+            return "data"
+        no_positional_args.exposed = True
+
+        def no_positional_args_kwargs(self, *args, **kwargs):
+            return "data"
+        no_positional_args_kwargs.exposed = True
+
+        def no_positional_kwargs(self, **kwargs):
+            return "data"
+        no_positional_kwargs.exposed = True
+
 
     class Status(Test):
         
@@ -349,7 +384,7 @@ def setup_server():
         
         def index(self):
             m = cherrypy.request.method
-            if m in defined_http_methods:
+            if m in defined_http_methods or m == "CONNECT":
                 return m
             
             if m == "LINK":
@@ -443,15 +478,12 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         
         self.getPage("/params/?thing=a&thing=b&thing=c")
         self.assertBody("['a', 'b', 'c']")
-        
+
         # Test friendly error message when given params are not accepted.
-        ignore = helper.webtest.ignored_exceptions
-        ignore.append(TypeError)
-        try:
-            self.getPage("/params/?notathing=meeting")
-            self.assertInBody("index() got an unexpected keyword argument 'notathing'")
-        finally:
-            ignore.pop()
+        self.getPage("/params/?notathing=meeting")
+        self.assertInBody("Missing parameters: thing")
+        self.getPage("/params/?thing=meeting&notathing=meeting")
+        self.assertInBody("Unexpected query string parameters: notathing")
         
         # Test "% HEX HEX"-encoded URL, param keys, and values
         self.getPage("/params/%d4%20%e3/cheese?Gruy%E8re=Bulgn%e9ville")
@@ -466,7 +498,82 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         # Test coordinates sent by <img ismap>
         self.getPage("/params/ismap?223,114")
         self.assertBody("Coordinates: 223, 114")
-    
+
+    def testParamErrors(self):
+
+        # test that all of the handlers work when given 
+        # the correct parameters in order to ensure that the
+        # errors below aren't coming from some other source.
+        for uri in (
+                '/paramerrors/one_positional?param1=foo',
+                '/paramerrors/one_positional_args?param1=foo',
+                '/paramerrors/one_positional_args/foo',
+                '/paramerrors/one_positional_args/foo/bar/baz',
+                '/paramerrors/one_positional_args_kwargs?param1=foo&param2=bar',
+                '/paramerrors/one_positional_args_kwargs/foo?param2=bar&param3=baz',
+                '/paramerrors/one_positional_args_kwargs/foo/bar/baz?param2=bar&param3=baz',
+                '/paramerrors/one_positional_kwargs?param1=foo&param2=bar&param3=baz',
+                '/paramerrors/one_positional_kwargs/foo?param4=foo&param2=bar&param3=baz',
+                '/paramerrors/no_positional',
+                '/paramerrors/no_positional_args/foo',
+                '/paramerrors/no_positional_args/foo/bar/baz',
+                '/paramerrors/no_positional_args_kwargs?param1=foo&param2=bar',
+                '/paramerrors/no_positional_args_kwargs/foo?param2=bar',
+                '/paramerrors/no_positional_args_kwargs/foo/bar/baz?param2=bar&param3=baz',
+                '/paramerrors/no_positional_kwargs?param1=foo&param2=bar',
+            ):
+            self.getPage(uri)
+            self.assertStatus(200)
+
+        # query string parameters are part of the URI, so if they are wrong
+        # for a particular handler, the status MUST be a 404.
+        for uri in (
+                '/paramerrors/one_positional',
+                '/paramerrors/one_positional?foo=foo',
+                '/paramerrors/one_positional/foo/bar/baz',
+                '/paramerrors/one_positional/foo?param1=foo',
+                '/paramerrors/one_positional/foo?param1=foo&param2=foo',
+                '/paramerrors/one_positional_args/foo?param1=foo&param2=foo',
+                '/paramerrors/one_positional_args/foo/bar/baz?param2=foo',
+                '/paramerrors/one_positional_args_kwargs/foo/bar/baz?param1=bar&param3=baz',
+                '/paramerrors/one_positional_kwargs/foo?param1=foo&param2=bar&param3=baz',
+                '/paramerrors/no_positional/boo',
+                '/paramerrors/no_positional?param1=foo',
+                '/paramerrors/no_positional_args/boo?param1=foo',
+                '/paramerrors/no_positional_kwargs/boo?param1=foo',
+            ):
+            self.getPage(uri)
+            self.assertStatus(404)
+
+        # if body parameters are wrong, a 400 must be returned.
+        for uri, body in (
+                ('/paramerrors/one_positional/foo', 'param1=foo',),
+                ('/paramerrors/one_positional/foo', 'param1=foo&param2=foo',),
+                ('/paramerrors/one_positional_args/foo', 'param1=foo&param2=foo',),
+                ('/paramerrors/one_positional_args/foo/bar/baz', 'param2=foo',),
+                ('/paramerrors/one_positional_args_kwargs/foo/bar/baz', 'param1=bar&param3=baz',),
+                ('/paramerrors/one_positional_kwargs/foo', 'param1=foo&param2=bar&param3=baz',),
+                ('/paramerrors/no_positional', 'param1=foo',),
+                ('/paramerrors/no_positional_args/boo', 'param1=foo',),
+            ):
+            self.getPage(uri, method='POST', body=body)
+            self.assertStatus(400)
+
+
+        # even if body parameters are wrong, if we get the uri wrong, then 
+        # it's a 404
+        for uri, body in (
+                ('/paramerrors/one_positional?param2=foo', 'param1=foo',),
+                ('/paramerrors/one_positional/foo/bar', 'param2=foo',),
+                ('/paramerrors/one_positional_args/foo/bar?param2=foo', 'param3=foo',),
+                ('/paramerrors/one_positional_kwargs/foo/bar', 'param2=bar&param3=baz',),
+                ('/paramerrors/no_positional?param1=foo', 'param2=foo',),
+                ('/paramerrors/no_positional_args/boo?param2=foo', 'param1=foo',),
+            ):
+            self.getPage(uri, method='POST', body=body)
+            self.assertStatus(404)
+
+
     def testStatus(self):
         self.getPage("/status/")
         self.assertBody('normal')
@@ -481,9 +588,10 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         msg = "Illegal response status from server (781 is out of range)."
         self.assertErrorPage(500, msg)
         
-        self.getPage("/status/unknown")
-        self.assertBody('funky')
-        self.assertStatus(431)
+        if not getattr(cherrypy.server, 'using_apache', False):
+            self.getPage("/status/unknown")
+            self.assertBody('funky')
+            self.assertStatus(431)
         
         self.getPage("/status/bad")
         self.assertStatus(500)
@@ -645,7 +753,8 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             self.getPage("/error/page_yield")
             self.assertErrorPage(500, pattern=valerr)
             
-            if cherrypy.server.protocol_version == "HTTP/1.0":
+            if (cherrypy.server.protocol_version == "HTTP/1.0" or
+                getattr(cherrypy.server, "using_apache", False)):
                 self.getPage("/error/page_streamed")
                 # Because this error is raised after the response body has
                 # started, the status should not change to an error status.
@@ -654,7 +763,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             else:
                 # Under HTTP/1.1, the chunked transfer-coding is used.
                 # The HTTP client will choke when the output is incomplete.
-                self.assertRaises(ValueError, self.getPage,
+                self.assertRaises((ValueError, IncompleteRead), self.getPage,
                                   "/error/page_streamed")
             
             # No traceback should be present
@@ -688,8 +797,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
                "IOError: [Errno 2] No such file or directory: 'nonexistent.html'")
         self.assertInBody(msg)
         
-        if (hasattr(self, 'harness') and
-            "modpython" in self.harness.__class__.__name__.lower()):
+        if getattr(cherrypy.server, "using_apache", False):
             pass
         else:
             # Test throw_errors (ticket #186).
@@ -853,7 +961,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
                      headers=[("Content-type", "application/json")])
         self.assertBody("application/json")
     
-    def testHTTPMethods(self):
+    def test_basic_HTTPMethods(self):
         helper.webtest.methods_with_bodies = ("POST", "PUT", "PROPFIND")
         
         # Test that all defined HTTP methods work.
@@ -938,6 +1046,14 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertBody('<h1>Choose your document</h1>\n<ul>\n</ul>')
         self.assertStatus(200)
     
+    def test_CONNECT_method(self):
+        if getattr(cherrypy.server, "using_apache", False):
+            print "skipped due to known Apache differences...",
+            return
+        
+        self.getPage("/method/", method="CONNECT")
+        self.assertBody("CONNECT")
+    
     def testFavicon(self):
         # favicon.ico is served by staticfile.
         icofilename = os.path.join(localDir, "../favicon.ico")
@@ -971,6 +1087,10 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             self.assertHeader('Set-Cookie', 'Last=Piranha;')
     
     def testMaxRequestSize(self):
+        if getattr(cherrypy.server, "using_apache", False):
+            print "skipped due to known Apache differences...",
+            return
+        
         for size in (500, 5000, 50000):
             self.getPage("/", headers=[('From', "x" * 500)])
             self.assertStatus(413)
@@ -1052,6 +1172,10 @@ Content-Type: text/plain
         self.assertBody('page1')
         self.getPage('/url/leaf?path_info=/page1&relative=True')
         self.assertBody('../page1')
+        self.getPage('/url/leaf?path_info=page1&relative=True')
+        self.assertBody('page1')
+        self.getPage('/url/leaf?path_info=leaf/page1&relative=True')
+        self.assertBody('leaf/page1')
         self.getPage('/url/leaf?path_info=../page1&relative=True')
         self.assertBody('../page1')
         self.getPage('/url/?path_info=other/../page1&relative=True')
