@@ -1,7 +1,9 @@
 from cherrypy.test import test
 test.prefer_parent_path()
 
+import sys
 import gzip, StringIO
+from httplib import IncompleteRead
 import cherrypy
 europoundUnicode = u'\x80\xa3'
 europoundUtf8 = u'\x80\xa3'.encode('utf-8')
@@ -152,7 +154,8 @@ class EncodingTests(helper.CPWebCase):
         # readable page, since 1) the gzip header is already set,
         # and 2) we may have already written some of the body.
         # The fix is to never stream yields when using gzip.
-        if cherrypy.server.protocol_version == "HTTP/1.0":
+        if (cherrypy.server.protocol_version == "HTTP/1.0" or
+            getattr(cherrypy.server, "using_apache", False)):
             self.getPage('/gzip/noshow_stream',
                          headers=[("Accept-Encoding", "gzip")])
             self.assertHeader('Content-Encoding', 'gzip')
@@ -160,10 +163,10 @@ class EncodingTests(helper.CPWebCase):
         else:
             # The wsgiserver will simply stop sending data, and the HTTP client
             # will error due to an incomplete chunk-encoded stream.
-            self.assertRaises(ValueError, self.getPage, '/gzip/noshow_stream',
+            self.assertRaises((ValueError, IncompleteRead), self.getPage,
+                              '/gzip/noshow_stream',
                               headers=[("Accept-Encoding", "gzip")])
 
-
 if __name__ == "__main__":
     setup_server()
     helper.testmain()
