@@ -1,4 +1,5 @@
 from cherrypy.test import test
+from cherrypy._cptree import Application
 test.prefer_parent_path()
 
 import cherrypy
@@ -219,16 +220,17 @@ class ObjectMappingTest(helper.CPWebCase):
             self.assertStatus('302 Found')
             self.assertHeader('Location', '%s/dir1/' % self.base())
             
-            # Test that we can use URL's which aren't all valid Python identifiers
-            # This should also test the %XX-unquoting of URL's.
-            self.getPage("/Von%20B%fclow?ID=14")
-            self.assertBody("ID is 14")
-            
-            # Test that %2F in the path doesn't get unquoted too early;
-            # that is, it should not be used to separate path components.
-            # See ticket #393.
-            self.getPage("/page%2Fname")
-            self.assertBody("default:('page/name',)")
+            if not getattr(cherrypy.server, "using_apache", False):
+                # Test that we can use URL's which aren't all valid Python identifiers
+                # This should also test the %XX-unquoting of URL's.
+                self.getPage("/Von%20B%fclow?ID=14")
+                self.assertBody("ID is 14")
+                
+                # Test that %2F in the path doesn't get unquoted too early;
+                # that is, it should not be used to separate path components.
+                # See ticket #393.
+                self.getPage("/page%2Fname")
+                self.assertBody("default:('page/name',)")
             
             self.getPage("/dir1/dir2/script_name")
             self.assertBody(url)
@@ -242,7 +244,7 @@ class ObjectMappingTest(helper.CPWebCase):
         self.script_name = ""
         
         # Test absoluteURI's in the Request-Line
-        self.getPage('http://127.0.0.1/')
+        self.getPage('http://%s:%s/' % (self.interface(), self.PORT))
         self.assertBody('world')
         
         # Test that the "isolated" app doesn't leak url's into the root app.
@@ -268,11 +270,10 @@ class ObjectMappingTest(helper.CPWebCase):
         self.getPage("/dir1/dir2/5/3/sir")
         self.assertBody("default for dir1, param is:('dir2', '5', '3', 'sir')")
         
-        # test that extra positional args raises an error.
-        # 500 for now, maybe 404 in the future.
+        # test that extra positional args raises an 404 Not Found
         # See http://www.cherrypy.org/ticket/733.
         self.getPage("/dir1/dir2/script_name/extra/stuff")
-        self.assertStatus(500)
+        self.assertStatus(404)
     
     def testExpose(self):
         # Test the cherrypy.expose function/decorator
@@ -324,6 +325,29 @@ class ObjectMappingTest(helper.CPWebCase):
         self.getPage("/app")
         self.assertBody("milk")
 
+    def testTreeMounting(self):
+
+        # When mounting an application instance, 
+        # we can't specify a script name in the call to mount.
+        a = Application(object(), '/somewhere')
+        self.assertRaises(ValueError, cherrypy.tree.mount, a, '/somewhereelse')
+
+        # When mounting an application instance, 
+        # we can't specify a script name in the call to mount.
+        a = Application(object(), '/somewhere')
+        try:
+            cherrypy.tree.mount(a, '/somewhere')
+        except ValueError:
+            self.fail("tree.mount must allow script_names which are the same")
+
+        try:
+            cherrypy.tree.mount(a)
+        except ValueError:
+            self.fail("cherrypy.tree.mount incorrectly raised a ValueError")
+
+
+
+
 
 if __name__ == "__main__":
     setup_server()
